//*****************************************************************************
//*****************************************************************************
//  FILENAME:   sensor.c
//  Version: 1.0, 2007/03/08
//  Generated by Andy Kiser
//
//  DESCRIPTION:  sensor specific functions 
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
//#define __DEBUG__
#define __FAST_TABLE__	//needs more memory, but works also with 14 bit
#define __CALC_ADC__	//Faster, but not possible for 14bit
#include <m8c.h>        // part specific constants and macros
#include "sensor.h"
#include "psocgpioint.h"
#include "spim.h"
#include "feature.h"
#include "parameters.h"
#include "types_n_macros.h"
#include "arith.h"
//----------------------------------------
// Sensor parameters
// Factory Defaults are defined here (NUM_xxx are defined in "sensor.h")
//----------------------------------------
const BYTE FactoryDefault_Param_8   [NUM_PARAM_8] =
{
	0x00,		// report mode
	0x00,		// LED mode
	0x00		// Input Mode
};

const WORD FactoryDefault_Param_16  [NUM_PARAM_16] = 
{
	50,			// report rate [ms]
	50,			// sample rate [ms]
	50000,		// sample rate [us]	(not used for this sensor)
	0,			// Offset Channel 1 [mV]
	0,			// Offset Channel 2 [mV]
	0,			// Offset Channel 3 [mV]
	0,			// Offset Channel 4 [mV]
	0,			// Offset Channel 5 [mV]
	0,			// Offset Channel 6 [mV]
	0,			// Offset Channel 7 [mV]
	0,			// Offset Channel 8 [mV]
};

const DWORD FactoryDefault_Param_32  [NUM_PARAM_32] =
{
	0x00000000	// currently no 32bit parameters
};

const tParam_168 FactoryDefault_Param_168 [NUM_PARAM_168] =
{
	"+/-10V ADC",	// user device name
	"Frame Number",		// user name Channel 0
	"Channel 0",		// user name Channel 1
	"Channel 1",		// user name Channel 2
	"Channel 2",		// user name Channel 3
	"Channel 3",		// user name Channel 4
	"Channel 4",		// user name Channel 5
	"Channel 5",		// user name Channel 6
	"Channel 6",		// user name Channel 7
	"Channel 7"			// user name Channel 8
};

//----------------------------------------
// ADC commands and constants
//----------------------------------------

#define SELECT_CH0		0x00
#define SELECT_CH1		0x10
#define SELECT_CH2		0x20
#define SELECT_CH3		0x30
#define SELECT_CH4		0x40
#define SELECT_CH5		0x50
#define SELECT_CH6		0x60
#define SELECT_CH7		0x70
#define WRITE_CFR		0xA0
#define SELECT_TEST1	0xB0
#define SELECT_TEST2	0xC0
#define SELECT_TEST3	0xD0
#define FIFO_READ		0xE0
#define HARDWARE_DEFULT	0xF0

//#define SINGLE_ENDED_MSB	0x0E
//#define SINGLE_ENDED_LSB	0x40
//#define DIFF_ENDED_MSB		0x0E
//#define DIFF_ENDED_LSB		0xC8
#define SINGLE_ENDED_MSB	0x0A
#define SINGLE_ENDED_LSB	0x00
#define DIFF_ENDED_MSB		0x0A
#define DIFF_ENDED_LSB		0x80

//----------------------------------------
// this module's global variables
//----------------------------------------
tReport Report;

//----------------------------------------
// constant lookup-table for ADC->physical value conversion
//----------------------------------------
#ifndef __CALC_ADC__
	#ifndef __FAST_TABLE__
		const int ADC2Volt_MSB[16] =	// Bits 11..8
		{		
			0	,
			1280	,
			2560	,
			3840	,
			5120	,
			6400	,
			7680	,
			8960	,
			10240	,
			11520	,
			12800	,
			14080	,
			15360	,
			16640	,
			17920	,
			19200
		};		
				
		const int ADC2Volt_LSB[256] =	// Bits 7..0
		{		
			0	,
			5	,
			10	,
			15	,
			20	,
			25	,
			30	,
			35	,
			40	,
			45	,
			50	,
			55	,
			60	,
			65	,
			70	,
			75	,
			80	,
			85	,
			90	,
			95	,
			100	,
			105	,
			110	,
			115	,
			120	,
			125	,
			130	,
			135	,
			140	,
			145	,
			150	,
			155	,
			160	,
			165	,
			170	,
			175	,
			180	,
			185	,
			190	,
			195	,
			200	,
			205	,
			210	,
			215	,
			220	,
			225	,
			230	,
			235	,
			240	,
			245	,
			250	,
			255	,
			260	,
			265	,
			270	,
			275	,
			280	,
			285	,
			290	,
			295	,
			300	,
			305	,
			310	,
			315	,
			320	,
			325	,
			330	,
			335	,
			340	,
			345	,
			350	,
			355	,
			360	,
			365	,
			370	,
			375	,
			380	,
			385	,
			390	,
			395	,
			400	,
			405	,
			410	,
			415	,
			420	,
			425	,
			430	,
			435	,
			440	,
			445	,
			450	,
			455	,
			460	,
			465	,
			470	,
			475	,
			480	,
			485	,
			490	,
			495	,
			500	,
			505	,
			510	,
			515	,
			520	,
			525	,
			530	,
			535	,
			540	,
			545	,
			550	,
			555	,
			560	,
			565	,
			570	,
			575	,
			580	,
			585	,
			590	,
			595	,
			600	,
			605	,
			610	,
			615	,
			620	,
			625	,
			630	,
			635	,
			640	,
			645	,
			650	,
			655	,
			660	,
			665	,
			670	,
			675	,
			680	,
			685	,
			690	,
			695	,
			700	,
			705	,
			710	,
			715	,
			720	,
			725	,
			730	,
			735	,
			740	,
			745	,
			750	,
			755	,
			760	,
			765	,
			770	,
			775	,
			780	,
			785	,
			790	,
			795	,
			800	,
			805	,
			810	,
			815	,
			820	,
			825	,
			830	,
			835	,
			840	,
			845	,
			850	,
			855	,
			860	,
			865	,
			870	,
			875	,
			880	,
			885	,
			890	,
			895	,
			900	,
			905	,
			910	,
			915	,
			920	,
			925	,
			930	,
			935	,
			940	,
			945	,
			950	,
			955	,
			960	,
			965	,
			970	,
			975	,
			980	,
			985	,
			990	,
			995	,
			1000	,
			1005	,
			1010	,
			1015	,
			1020	,
			1025	,
			1030	,
			1035	,
			1040	,
			1045	,
			1050	,
			1055	,
			1060	,
			1065	,
			1070	,
			1075	,
			1080	,
			1085	,
			1090	,
			1095	,
			1100	,
			1105	,
			1110	,
			1115	,
			1120	,
			1125	,
			1130	,
			1135	,
			1140	,
			1145	,
			1150	,
			1155	,
			1160	,
			1165	,
			1170	,
			1175	,
			1180	,
			1185	,
			1190	,
			1195	,
			1200	,
			1205	,
			1210	,
			1215	,
			1220	,
			1225	,
			1230	,
			1235	,
			1240	,
			1245	,
			1250	,
			1255	,
			1260	,
			1265	,
			1270	,
			1275
		};		
	#else //__FAST_TABLE__
		const int ADC2VoltF_MSB[256] =	// Bits 11..4
		{
			0	,
			80	,
			160	,
			240	,
			320	,
			400	,
			480	,
			560	,
			640	,
			720	,
			800	,
			880	,
			960	,
			1040	,
			1120	,
			1200	,
			1280	,
			1360	,
			1440	,
			1520	,
			1600	,
			1680	,
			1760	,
			1840	,
			1920	,
			2000	,
			2080	,
			2160	,
			2240	,
			2320	,
			2400	,
			2480	,
			2560	,
			2640	,
			2720	,
			2800	,
			2880	,
			2960	,
			3040	,
			3120	,
			3200	,
			3280	,
			3360	,
			3440	,
			3520	,
			3600	,
			3680	,
			3760	,
			3840	,
			3920	,
			4000	,
			4080	,
			4160	,
			4240	,
			4320	,
			4400	,
			4480	,
			4560	,
			4640	,
			4720	,
			4800	,
			4880	,
			4960	,
			5040	,
			5120	,
			5200	,
			5280	,
			5360	,
			5440	,
			5520	,
			5600	,
			5680	,
			5760	,
			5840	,
			5920	,
			6000	,
			6080	,
			6160	,
			6240	,
			6320	,
			6400	,
			6480	,
			6560	,
			6640	,
			6720	,
			6800	,
			6880	,
			6960	,
			7040	,
			7120	,
			7200	,
			7280	,
			7360	,
			7440	,
			7520	,
			7600	,
			7680	,
			7760	,
			7840	,
			7920	,
			8000	,
			8080	,
			8160	,
			8240	,
			8320	,
			8400	,
			8480	,
			8560	,
			8640	,
			8720	,
			8800	,
			8880	,
			8960	,
			9040	,
			9120	,
			9200	,
			9280	,
			9360	,
			9440	,
			9520	,
			9600	,
			9680	,
			9760	,
			9840	,
			9920	,
			10000	,
			10080	,
			10160	,
			10240	,
			10320	,
			10400	,
			10480	,
			10560	,
			10640	,
			10720	,
			10800	,
			10880	,
			10960	,
			11040	,
			11120	,
			11200	,
			11280	,
			11360	,
			11440	,
			11520	,
			11600	,
			11680	,
			11760	,
			11840	,
			11920	,
			12000	,
			12080	,
			12160	,
			12240	,
			12320	,
			12400	,
			12480	,
			12560	,
			12640	,
			12720	,
			12800	,
			12880	,
			12960	,
			13040	,
			13120	,
			13200	,
			13280	,
			13360	,
			13440	,
			13520	,
			13600	,
			13680	,
			13760	,
			13840	,
			13920	,
			14000	,
			14080	,
			14160	,
			14240	,
			14320	,
			14400	,
			14480	,
			14560	,
			14640	,
			14720	,
			14800	,
			14880	,
			14960	,
			15040	,
			15120	,
			15200	,
			15280	,
			15360	,
			15440	,
			15520	,
			15600	,
			15680	,
			15760	,
			15840	,
			15920	,
			16000	,
			16080	,
			16160	,
			16240	,
			16320	,
			16400	,
			16480	,
			16560	,
			16640	,
			16720	,
			16800	,
			16880	,
			16960	,
			17040	,
			17120	,
			17200	,
			17280	,
			17360	,
			17440	,
			17520	,
			17600	,
			17680	,
			17760	,
			17840	,
			17920	,
			18000	,
			18080	,
			18160	,
			18240	,
			18320	,
			18400	,
			18480	,
			18560	,
			18640	,
			18720	,
			18800	,
			18880	,
			18960	,
			19040	,
			19120	,
			19200	,
			19280	,
			19360	,
			19440	,
			19520	,
			19600	,
			19680	,
			19760	,
			19840	,
			19920	,
			20000	,
			20080	,
			20160	,
			20240	,
			20320	,
			20400
		};		
				
		const int ADC2VoltF_LSB[256] =	// Bits 3..0
		{
			0	,
			0	,
			1	,
			1	,
			1	,
			2	,
			2	,
			2	,
			3	,
			3	,
			3	,
			3	,
			4	,
			4	,
			4	,
			5	,
			5	,
			5	,
			6	,
			6	,
			6	,
			7	,
			7	,
			7	,
			8	,
			8	,
			8	,
			8	,
			9	,
			9	,
			9	,
			10	,
			10	,
			10	,
			11	,
			11	,
			11	,
			12	,
			12	,
			12	,
			13	,
			13	,
			13	,
			13	,
			14	,
			14	,
			14	,
			15	,
			15	,
			15	,
			16	,
			16	,
			16	,
			17	,
			17	,
			17	,
			18	,
			18	,
			18	,
			18	,
			19	,
			19	,
			19	,
			20	,
			20	,
			20	,
			21	,
			21	,
			21	,
			22	,
			22	,
			22	,
			23	,
			23	,
			23	,
			23	,
			24	,
			24	,
			24	,
			25	,
			25	,
			25	,
			26	,
			26	,
			26	,
			27	,
			27	,
			27	,
			28	,
			28	,
			28	,
			28	,
			29	,
			29	,
			29	,
			30	,
			30	,
			30	,
			31	,
			31	,
			31	,
			32	,
			32	,
			32	,
			33	,
			33	,
			33	,
			33	,
			34	,
			34	,
			34	,
			35	,
			35	,
			35	,
			36	,
			36	,
			36	,
			37	,
			37	,
			37	,
			38	,
			38	,
			38	,
			38	,
			39	,
			39	,
			39	,
			40	,
			40	,
			40	,
			41	,
			41	,
			41	,
			42	,
			42	,
			42	,
			43	,
			43	,
			43	,
			43	,
			44	,
			44	,
			44	,
			45	,
			45	,
			45	,
			46	,
			46	,
			46	,
			47	,
			47	,
			47	,
			48	,
			48	,
			48	,
			48	,
			49	,
			49	,
			49	,
			50	,
			50	,
			50	,
			51	,
			51	,
			51	,
			52	,
			52	,
			52	,
			53	,
			53	,
			53	,
			53	,
			54	,
			54	,
			54	,
			55	,
			55	,
			55	,
			56	,
			56	,
			56	,
			57	,
			57	,
			57	,
			58	,
			58	,
			58	,
			58	,
			59	,
			59	,
			59	,
			60	,
			60	,
			60	,
			61	,
			61	,
			61	,
			62	,
			62	,
			62	,
			63	,
			63	,
			63	,
			63	,
			64	,
			64	,
			64	,
			65	,
			65	,
			65	,
			66	,
			66	,
			66	,
			67	,
			67	,
			67	,
			68	,
			68	,
			68	,
			68	,
			69	,
			69	,
			69	,
			70	,
			70	,
			70	,
			71	,
			71	,
			71	,
			72	,
			72	,
			72	,
			73	,
			73	,
			73	,
			73	,
			74	,
			74	,
			74	,
			75	,
			75	,
			75	,
			76	,
			76	,
			76	,
			77	,
			77	,
			77	,
			78	,
			78	,
			78	,
			78	,
			79	,
			79	,
			79	,
			80
		};		
	#endif	//__FAST_TABLE__
#endif //__CALC_ADC__
//----------------------------------------
// type definitions
//----------------------------------------

typedef enum 
{	
	eReset,
	eIdle,
} tFSM_Measure;
/*
typedef struct
{
	union uBigEndian Axis[3];
} tWordVector3D;
*/

//----------------------------------------
// variables
//----------------------------------------
tFSM_Measure	FSM;
WORD			Countdown_1ms;
WORD			OffsetVoltage[8];
BOOL			NewSample = FALSE;

// store original drivemodes during suspend
BYTE			DM0_2, DM0_1, DM0_0;
BYTE			DM2_2, DM2_1, DM2_0;
BYTE			Mode;
extern BYTE		SOF_Flag; // needed to compensate for FSM processing duration >1ms
//----------------------------------------
// function prototypes
//----------------------------------------
/*
static WORD sqrt(DWORD arg);
static int Calculate_g(union uBigEndian ADC);
static WORD Atan2 (int x, int y);
*/
WORD ADC_RxTx(BYTE MSB,BYTE LSB);
static WORD Calculate_v(WORD ADC, BYTE Channel);
static WORD Calculate_dv(WORD ADC, WORD *NegChan, BYTE Channel);
void Calculate_Offset(void);

//-----------------------------------------------------------------------------
// Initializes the Sensor 
//-----------------------------------------------------------------------------
void Sensor_Start ()
{
	BYTE InputMode;
	WORD i;
	FSM = eIdle;
	// Initialize SPI to mode 3 as it is neccessary to communicate with the SMB365 sensor
	SPIM_Start(SPIM_SPIM_MODE_2 | SPIM_SPIM_MSB_FIRST);
	// nCS idle state = 1
	GPIO_HIGH(nCS)
	// turn on power for the ADC
	GPIO_LOW(OFF_nON)
	Countdown_1ms = 0;
	Get_Param_8(PAR8_INPUT_MODE, VOLATILE, &InputMode);
	//while(Countdown_1ms >0) Countdown_1ms--;
	if (InputMode)
		ADC_RxTx(WRITE_CFR | DIFF_ENDED_MSB,DIFF_ENDED_LSB);
	else
		ADC_RxTx(WRITE_CFR | SINGLE_ENDED_MSB,SINGLE_ENDED_LSB);
	ADC_RxTx(SELECT_CH0,0);
	Calculate_Offset();
}
//-----------------------------------------------------------------------------
// Prepare Sensor for Suspend
//-----------------------------------------------------------------------------
void Sensor_Suspend()
{
	// turn off power for the sensor
	GPIO_HIGH(OFF_nON)
		
	// store original drivemodes during  suspend
	DM0_2 = PRT0DM2;
	DM0_1 = PRT0DM1;
	DM0_0 = PRT0DM0;
	DM2_2 = PRT2DM2;
	DM2_1 = PRT2DM1;
	DM2_0 = PRT2DM0;
	
	// set all I/Os to/from the sensor to High-Z Analog (=110), 
	// except the OFF_nON signal. 
	// OFF_nON is on Port0, all other signals on Port2
	PRT0DM2 |= ~OFF_nON_MASK;
	PRT0DM1 |= ~OFF_nON_MASK;
	PRT0DM0 &=  OFF_nON_MASK;
	PRT2DM2  = 0xFF;
	PRT2DM1  = 0xFF;
	PRT2DM0  = 0x00;
}
//-----------------------------------------------------------------------------
// Resume Sensor from Suspend mode
//-----------------------------------------------------------------------------
void Sensor_Resume()
{
	// restore drivemodes for GPIOs to/from the sensor
	PRT2DM2 = DM2_2;
	PRT2DM1 = DM2_1;
	PRT2DM0 = DM2_0;

	PRT0DM2 = DM0_2;
	PRT0DM1 = DM0_1;
	PRT0DM0 = DM0_0;

	// start up sensor again
	Sensor_Start();
}

//-----------------------------------------------------------------------------
// Read one register byte from the sensor
// The function assumes the following (idle) conditions on entry:
// 		CSn == 1
// 		SPI-Mode == 0
//-----------------------------------------------------------------------------
/*
WORD ADC_RxTx(BYTE MSB,BYTE LSB)
{
	WORD ReturnValue = 0;
	WORD SendValue = (MSB<<8) + LSB;
	WORD i, n;
	//GPIO_LOW(FS)
	//for (i=0;i<=100;i++);//Wait
	GPIO_LOW(nCS)
	for (i=0;i<=0;i++);//Wait
	//GPIO_HIGH(FS)
	for (n=0;n<=15;n++)
	{
		GPIO_HIGH(SCK2)	//Clock to High
		if(SendValue & 0x8000) {GPIO_HIGH(MOSI2)} else {GPIO_LOW(MOSI2)}	//Write new value
		SendValue <<=1;
		for (i=0;i<=0;i++);//Wait
		
		GPIO_LOW(SCK2)	//Clock to Low
		if(MISO2_Data_ADDR & MISO2_MASK) ReturnValue |=1;		//Read Value
		ReturnValue <<=1;
		for (i=0;i<=0;i++);//Wait
	}

	//GPIO_LOW(nCSSTART)
	//for (i=0;i<=10;i++);//Wait
	//GPIO_HIGH(nCSSTART)
	//for (i=0;i<=0;i++);//Wait

	//for (n=0;n<=31;n++)
	//{
	//	GPIO_HIGH(SCK2)	//Clock to High
	//	for (i=0;i<=0;i++);//Wait
	//	GPIO_LOW(SCK2)	//Clock to Low
	//	for (i=0;i<=0;i++);//Wait
	//}

	GPIO_HIGH(nCS)
	
	
	return ReturnValue;
}
*/

WORD ADC_RxTx(BYTE MSB,BYTE LSB)
{
	union uBigEndian ReturnValue;
	GPIO_LOW(nCS)

	//SPIM_bReadRxData(); 	  // -- dummy read to clear RX_BUFFER_FULL flag
	SPIM_SendTxData(MSB); // Transmit Address
	while( !( SPIM_bReadStatus() & SPIM_SPIM_RX_BUFFER_FULL ) );   	
	ReturnValue.BE.Hi = SPIM_bReadRxData();

	// -- Receive Data Byte
	SPIM_SendTxData(LSB);
	while( !( SPIM_bReadStatus() & SPIM_SPIM_RX_BUFFER_FULL ) );   	
	ReturnValue.BE.Lo = SPIM_bReadRxData();
	
	GPIO_HIGH(nCS)
	
	return ReturnValue.w;
}


//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// cannot be handled with the standard functionality in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//
//	return Value: number of Parameter bytes (1-31) that are 
//				  returned in the IN report
//-----------------------------------------------------------------------------

BYTE Sensor_Specific_Feature(void)
{
	return 0;	
}

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// sets / gets a 8/16/32/168bit parameter in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//-----------------------------------------------------------------------------
void Sensor_Specific_Parameter(BYTE *NumberOfReturnedBytes)
{
	Calculate_Offset();
	// currently there are no specific parameters
}

//-----------------------------------------------------------------------------
// void Sensor_FSM_Measure(void)
//
//	finite state machine to control the measurement process for 
//  the Sensor
//
//  IMPORTANT: 	call this routine once every millisecond. Otherwise the timing
//				parameters cannot be met.
//-----------------------------------------------------------------------------

void Sensor_FSM_Measure() 
{
	BYTE InputMode;
	
	// decrement millisecond counter
	if (Countdown_1ms != 0)
		Countdown_1ms--;

	// process actual state machine
	switch (FSM)
	{
		case eIdle:
			if (Countdown_1ms == 0)
			{
				// reset samplerate counter
				Get_Param_16(PAR16_SAMPLERATE_MS, VOLATILE, &Countdown_1ms);
				Get_Param_8(PAR8_INPUT_MODE, VOLATILE, &InputMode);
				NewSample = TRUE;
				if (InputMode)
				{
					ADC_RxTx(SELECT_CH0,0);
					Report.Ch0 = Calculate_dv(ADC_RxTx(SELECT_CH2,0),&Report.Ch1,0);
					Report.Ch2 = Calculate_dv(ADC_RxTx(SELECT_CH4,0),&Report.Ch3,1);
					Report.Ch4 = Calculate_dv(ADC_RxTx(SELECT_CH6,0),&Report.Ch5,2);
					Report.Ch6 = Calculate_dv(ADC_RxTx(WRITE_CFR | DIFF_ENDED_MSB, DIFF_ENDED_LSB),&Report.Ch7,3);
				}
				else
				{
					ADC_RxTx(SELECT_CH0,0);
					Report.Ch0 = Calculate_v(ADC_RxTx(SELECT_CH1,0),0);
					Report.Ch1 = Calculate_v(ADC_RxTx(SELECT_CH2,0),1);
					Report.Ch2 = Calculate_v(ADC_RxTx(SELECT_CH3,0),2);
					Report.Ch3 = Calculate_v(ADC_RxTx(SELECT_CH4,0),3);
					Report.Ch4 = Calculate_v(ADC_RxTx(SELECT_CH5,0),4);
					Report.Ch5 = Calculate_v(ADC_RxTx(SELECT_CH6,0),5);
					Report.Ch6 = Calculate_v(ADC_RxTx(SELECT_CH7,0),6);
					Report.Ch7 = Calculate_v(ADC_RxTx(WRITE_CFR | SINGLE_ENDED_MSB, SINGLE_ENDED_LSB),7);
				}
			}
			break;
	}		
}
//-----------------------------------------------------------------------------
// calculate voltage, out of the raw ADC value
//-----------------------------------------------------------------------------
#ifndef __CALC_ADC__
	#ifndef __FAST_TABLE__
		static WORD Calculate_v(WORD ADC, BYTE Channel)
		{
			union uBigEndian result, retVal;
			BYTE ADC_LSB,ADC_MSB;
			ADC_LSB = ADC >> 4;
			ADC_MSB = ADC >>12;
			result.i = ADC2Volt_MSB[ADC_MSB]+ ADC2Volt_LSB[ADC_LSB] - OffsetVoltage[Channel];
			retVal.LE.Hi = result.BE.Hi;
			retVal.LE.Lo = result.BE.Lo;
			return retVal.w;
		}
	#else	//__FAST_TABLE__
		static WORD Calculate_v(WORD ADC, BYTE Channel)
		{
			union uBigEndian result, retVal;
			result.w = ADC;
			result.i = ADC2VoltF_MSB[result.BE.Hi]+ ADC2VoltF_LSB[result.BE.Lo] - OffsetVoltage[Channel];
			retVal.LE.Hi = result.BE.Hi;
			retVal.LE.Lo = result.BE.Lo;
			return retVal.w;
		}
	#endif //__FAST_TABLE__
#else //__CALC_ADC__
	static WORD Calculate_v(WORD ADC, BYTE Channel)
	{
		union uBigEndian result, retVal;
		retVal.w = (ADC>>2);
		result.w = retVal.w + (retVal.w>>2) - OffsetVoltage[Channel];
		retVal.LE.Hi = result.BE.Hi;
		retVal.LE.Lo = result.BE.Lo;
		return retVal.w;
	}
#endif //__CALC_ADC__
//-----------------------------------------------------------------------------
// calculate differental voltage, out of the raw ADC value
//-----------------------------------------------------------------------------

#ifndef __CALC_ADC__
	#ifndef __FAST_TABLE__
		static WORD Calculate_dv(WORD ADC, WORD *NegChan, BYTE Channel)
		{
			union uBigEndian result, retVal;
			BYTE ADC_LSB,ADC_MSB;
			ADC_LSB = ADC >> 4;
			ADC_MSB = ADC >>12;
			result.i = OffsetVoltage[Channel] - ADC2Volt_MSB[ADC_MSB] - ADC2Volt_LSB[ADC_LSB];
			retVal.LE.Hi = result.BE.Hi;
			retVal.LE.Lo = result.BE.Lo;
			*NegChan = retVal.w;
			result.i = 0-result.i;
			retVal.LE.Hi = result.BE.Hi;
			retVal.LE.Lo = result.BE.Lo;
			return retVal.w;
		}
	#else	//__FAST_TABLE__
		static WORD Calculate_dv(WORD ADC, WORD *NegChan, BYTE Channel)
		{
			union uBigEndian result, retVal;
			result.w = ADC;
			result.i = OffsetVoltage[Channel] - ADC2VoltF_MSB[result.BE.Hi] - ADC2VoltF_LSB[result.BE.Lo];
			retVal.LE.Hi = result.BE.Hi;
			retVal.LE.Lo = result.BE.Lo;
			*NegChan = retVal.w;
			result.i = 0-result.i;
			retVal.LE.Hi = result.BE.Hi;
			retVal.LE.Lo = result.BE.Lo;
			return retVal.w;
		}
	#endif //__FAST_TABLE__
#else //__CALC_ADC__
	static WORD Calculate_dv(WORD ADC, WORD *NegChan, BYTE Channel)
	{
		union uBigEndian result, retVal;
		retVal.w = (ADC>>2);
		result.w = OffsetVoltage[Channel] - retVal.w - (retVal.w>>2);
		retVal.LE.Hi = result.BE.Hi;
		retVal.LE.Lo = result.BE.Lo;
		*NegChan = retVal.w;
		result.i = 0-result.i;
		retVal.LE.Hi = result.BE.Hi;
		retVal.LE.Lo = result.BE.Lo;
		return retVal.w;
	}
#endif //__CALC_ADC__

//-----------------------------------------------------------------------------
// calculates the offset values out of the offset settings
//-----------------------------------------------------------------------------
void Calculate_Offset()
{	
	BYTE i;
	WORD Offset;
	for(i=0;i<8;i++)
	{
		Get_Param_16(PAR16_OFFSET_CH1 + i, VOLATILE, &Offset);
		OffsetVoltage[i] = 10240 - Offset;
	}
}


//-----------------------------------------------------------------------------
void Sensor_Debug(void)
{
/* currently no debug function */
}